@@ -27,7 +27,7 @@
 #include <config.h>
 #endif
 
-#ifdef WITH_MP4
+#include <climits>
 
 #include <tdebug.h>
 #include <tstring.h>
@@ -35,9 +35,10 @@
 
 using namespace TagLib;
 
-const char *MP4::Atom::containers[10] = {
+const char *MP4::Atom::containers[11] = {
     "moov", "udta", "mdia", "meta", "ilst",
     "stbl", "minf", "moof", "traf", "trak",
+    "stsd"
 };
 
 MP4::Atom::Atom(File *file)
@@ -53,22 +54,27 @@ MP4::Atom::Atom(File *file)
     return;
   }
 
-  length = header.mid(0, 4).toUInt();
+  length = header.toUInt();
 
-  if (length == 1) {
-    long long longLength = file->readBlock(8).toLongLong();
-    if (longLength >= 8 && longLength <= 0xFFFFFFFF) {
-        // The atom has a 64-bit length, but it's actually a 32-bit value
-        length = (long)longLength;
+  if(length == 1) {
+    const long long longLength = file->readBlock(8).toLongLong();
+    if(sizeof(long) == sizeof(long long)) {
+      length = static_cast<long>(longLength);
     }
     else {
+      if(longLength <= LONG_MAX) {
+        // The atom has a 64-bit length, but it's actually a 31-bit value
+        length = static_cast<long>(longLength);
+      }
+      else {
         debug("MP4: 64-bit atoms are not supported");
         length = 0;
         file->seek(0, File::End);
         return;
+      }
     }
   }
-  if (length < 8) {
+  if(length < 8) {
     debug("MP4: Invalid atom size");
     length = 0;
     file->seek(0, File::End);
@@ -82,6 +88,9 @@ MP4::Atom::Atom(File *file)
       if(name == "meta") {
         file->seek(4, File::Current);
       }
+      else if(name == "stsd") {
+        file->seek(8, File::Current);
+      }
       while(file->tell() < offset + length) {
         MP4::Atom *child = new MP4::Atom(file);
         children.append(child);
@@ -194,4 +203,3 @@ MP4::Atoms::path(const char *name1, const char *name2, const char *name3, const
   return path;
 }
 
-#endif
