@@ -25,56 +25,9 @@
 
 #include <tstring.h>
 #include <tdebug.h>
-#include <cmath>
-// ldexp is a c99 function, which might not be defined in <cmath>
-// so we pull in math.h too and hope it does the right (wrong) thing
-// wrt. c99 functions in C++
-#include <math.h>
-
+#include "aifffile.h"
 #include "aiffproperties.h"
 
-////////////////////////////////////////////////////////////////////////////////
-// nasty 80-bit float helpers
-////////////////////////////////////////////////////////////////////////////////
-
-#define UnsignedToFloat(u) (((double)((long)(u - 2147483647L - 1))) + 2147483648.0)
-
-static double ConvertFromIeeeExtended(unsigned char *bytes)
-{
-  double f;
-  int expon;
-  unsigned long hiMant, loMant;
-    
-  expon  = ((bytes[0] & 0x7F) << 8) | (bytes[1] & 0xFF);
-
-  hiMant = ((unsigned long)(bytes[2] & 0xFF) << 24) |
-           ((unsigned long)(bytes[3] & 0xFF) << 16) |
-           ((unsigned long)(bytes[4] & 0xFF) << 8)  |
-           ((unsigned long)(bytes[5] & 0xFF));
-
-  loMant = ((unsigned long)(bytes[6] & 0xFF) << 24) |
-           ((unsigned long)(bytes[7] & 0xFF) << 16) |
-           ((unsigned long)(bytes[8] & 0xFF) << 8)  |
-           ((unsigned long)(bytes[9] & 0xFF));
-
-  if (expon == 0 && hiMant == 0 && loMant == 0)
-    f = 0;
-  else {
-    if(expon == 0x7FFF) /* Infinity or NaN */
-      f = HUGE_VAL;
-    else {
-      expon -= 16383;
-      f  = ldexp(UnsignedToFloat(hiMant), expon -= 31);
-      f += ldexp(UnsignedToFloat(loMant), expon -= 32);
-    }
-  }
-
-  if(bytes[0] & 0x80)
-    return -f;
-  else
-    return f;
-}
-
 using namespace TagLib;
 
 class RIFF::AIFF::Properties::PropertiesPrivate
@@ -85,26 +38,37 @@ public:
     bitrate(0),
     sampleRate(0),
     channels(0),
-    sampleWidth(0)
-  {
-
-  }
+    bitsPerSample(0),
+    sampleFrames(0) {}
 
   int length;
   int bitrate;
   int sampleRate;
   int channels;
-  int sampleWidth;
+  int bitsPerSample;
+
+  ByteVector compressionType;
+  String compressionName;
+
+  uint sampleFrames;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-RIFF::AIFF::Properties::Properties(const ByteVector &data, ReadStyle style) : AudioProperties(style)
+RIFF::AIFF::Properties::Properties(const ByteVector &, ReadStyle style) :
+  AudioProperties(style),
+  d(new PropertiesPrivate())
 {
-  d = new PropertiesPrivate;
-  read(data);
+  debug("RIFF::AIFF::Properties::Properties() - This constructor is no longer used.");
+}
+
+RIFF::AIFF::Properties::Properties(File *file, ReadStyle style) :
+  AudioProperties(style),
+  d(new PropertiesPrivate())
+{
+  read(file);
 }
 
 RIFF::AIFF::Properties::~Properties()
@@ -113,6 +77,16 @@ RIFF::AIFF::Properties::~Properties()
 }
 
 int RIFF::AIFF::Properties::length() const
+{
+  return lengthInSeconds();
+}
+
+int RIFF::AIFF::Properties::lengthInSeconds() const
+{
+  return d->length / 1000;
+}
+
+int RIFF::AIFF::Properties::lengthInMilliseconds() const
 {
   return d->length;
 }
@@ -132,22 +106,86 @@ int RIFF::AIFF::Properties::channels() const
   return d->channels;
 }
 
+int RIFF::AIFF::Properties::bitsPerSample() const
+{
+  return d->bitsPerSample;
+}
+
 int RIFF::AIFF::Properties::sampleWidth() const
 {
-  return d->sampleWidth;
+  return bitsPerSample();
+}
+
+TagLib::uint RIFF::AIFF::Properties::sampleFrames() const
+{
+  return d->sampleFrames;
+}
+
+bool RIFF::AIFF::Properties::isAiffC() const
+{
+  return (!d->compressionType.isEmpty());
+}
+
+ByteVector RIFF::AIFF::Properties::compressionType() const
+{
+  return d->compressionType;
+}
+
+String RIFF::AIFF::Properties::compressionName() const
+{
+  return d->compressionName;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // private members
 ////////////////////////////////////////////////////////////////////////////////
 
-void RIFF::AIFF::Properties::read(const ByteVector &data)
+void RIFF::AIFF::Properties::read(File *file)
 {
-  d->channels       = data.mid(0, 2).toShort();
-  uint sampleFrames = data.mid(2, 4).toUInt();
-  d->sampleWidth    = data.mid(6, 2).toShort();
-  double sampleRate = ConvertFromIeeeExtended(reinterpret_cast<unsigned char *>(data.mid(8, 10).data()));
-  d->sampleRate     = sampleRate;
-  d->bitrate        = (sampleRate * d->sampleWidth * d->channels) / 1000.0;
-  d->length         = d->sampleRate > 0 ? sampleFrames / d->sampleRate : 0;
+  ByteVector data;
+  uint streamLength = 0;
+  for(uint i = 0; i < file->chunkCount(); i++) {
+    const ByteVector name = file->chunkName(i);
+    if(name == "COMM") {
+      if(data.isEmpty())
+        data = file->chunkData(i);
+      else
+        debug("RIFF::AIFF::Properties::read() - Duplicate 'COMM' chunk found.");
+    }
+    else if(name == "SSND") {
+      if(streamLength == 0)
+        streamLength = file->chunkDataSize(i) + file->chunkPadding(i);
+      else
+        debug("RIFF::AIFF::Properties::read() - Duplicate 'SSND' chunk found.");
+    }
+  }
+
+  if(data.size() < 18) {
+    debug("RIFF::AIFF::Properties::read() - 'COMM' chunk not found or too short.");
+    return;
+  }
+
+  if(streamLength == 0) {
+    debug("RIFF::AIFF::Properties::read() - 'SSND' chunk not found.");
+    return;
+  }
+
+  d->channels      = data.toShort(0U);
+  d->sampleFrames  = data.toUInt(2U);
+  d->bitsPerSample = data.toShort(6U);
+
+  const long double sampleRate = data.toFloat80BE(8);
+  if(sampleRate >= 1.0)
+    d->sampleRate = static_cast<int>(sampleRate + 0.5);
+
+  if(d->sampleFrames > 0 && d->sampleRate > 0) {
+    const double length = d->sampleFrames * 1000.0 / d->sampleRate;
+    d->length  = static_cast<int>(length + 0.5);
+    d->bitrate = static_cast<int>(streamLength * 8.0 / length + 0.5);
+  }
+
+  if(data.size() >= 23) {
+    d->compressionType = data.mid(18, 4);
+    d->compressionName = String(data.mid(23, static_cast<uchar>(data[22])), String::Latin1);
+  }
 }
