@@ -22,9 +22,9 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <cppunit/extensions/HelperMacros.h>
 #include <tstring.h>
 #include <string.h>
+#include <cppunit/extensions/HelperMacros.h>
 
 using namespace std;
 using namespace TagLib;
@@ -38,9 +38,14 @@ class TestString : public CppUnit::TestFixture
   CPPUNIT_TEST(testUTF16Decode);
   CPPUNIT_TEST(testUTF16DecodeInvalidBOM);
   CPPUNIT_TEST(testUTF16DecodeEmptyWithBOM);
+  CPPUNIT_TEST(testSurrogatePair);
   CPPUNIT_TEST(testAppendCharDetach);
   CPPUNIT_TEST(testAppendStringDetach);
   CPPUNIT_TEST(testToInt);
+  CPPUNIT_TEST(testSubstr);
+  CPPUNIT_TEST(testNewline);
+  CPPUNIT_TEST(testEncode);
+  CPPUNIT_TEST(testIterator);
   CPPUNIT_TEST_SUITE_END();
 
 public:
@@ -53,6 +58,14 @@ public:
 
     char str[] = "taglib string";
     CPPUNIT_ASSERT(strcmp(s.toCString(), str) == 0);
+    CPPUNIT_ASSERT(s == "taglib string");
+    CPPUNIT_ASSERT(s != "taglib STRING");
+    CPPUNIT_ASSERT(s != "taglib");
+    CPPUNIT_ASSERT(s != "taglib string taglib");
+    CPPUNIT_ASSERT(s == L"taglib string");
+    CPPUNIT_ASSERT(s != L"taglib STRING");
+    CPPUNIT_ASSERT(s != L"taglib");
+    CPPUNIT_ASSERT(s != L"taglib string taglib");
 
     String unicode("José Carlos", String::UTF8);
     CPPUNIT_ASSERT(strcmp(unicode.toCString(), "Jos\xe9 Carlos") == 0);
@@ -60,9 +73,33 @@ public:
     String latin = "Jos\xe9 Carlos";
     CPPUNIT_ASSERT(strcmp(latin.toCString(true), "José Carlos") == 0);
 
+    String c;
+    c = "1";
+    CPPUNIT_ASSERT(c == L"1");
+
+    c = L'\u4E00';
+    CPPUNIT_ASSERT(c == L"\u4E00");
+
     String unicode2(unicode.to8Bit(true), String::UTF8);
     CPPUNIT_ASSERT(unicode == unicode2);
 
+    String unicode3(L"\u65E5\u672C\u8A9E");
+    CPPUNIT_ASSERT(*(unicode3.toCWString() + 1) == L'\u672C');
+
+    String unicode4(L"\u65e5\u672c\u8a9e", String::UTF16BE);
+    CPPUNIT_ASSERT(unicode4[1] == L'\u672c');
+
+    String unicode5(L"\u65e5\u672c\u8a9e", String::UTF16LE);
+    CPPUNIT_ASSERT(unicode5[1] == L'\u2c67');
+
+    std::wstring stduni = L"\u65e5\u672c\u8a9e";
+
+    String unicode6(stduni, String::UTF16BE);
+    CPPUNIT_ASSERT(unicode6[1] == L'\u672c');
+
+    String unicode7(stduni, String::UTF16LE);
+    CPPUNIT_ASSERT(unicode7[1] == L'\u2c67');
+
     CPPUNIT_ASSERT(strcmp(String::number(0).toCString(), "0") == 0);
     CPPUNIT_ASSERT(strcmp(String::number(12345678).toCString(), "12345678") == 0);
     CPPUNIT_ASSERT(strcmp(String::number(-12345678).toCString(), "-12345678") == 0);
@@ -83,12 +120,10 @@ public:
     CPPUNIT_ASSERT(memcmp(String("foo").data(String::Latin1).data(), "foo", 3) == 0);
     CPPUNIT_ASSERT(memcmp(String("f").data(String::Latin1).data(), "f", 1) == 0);
 
-    ByteVector utf16 = unicode.data(String::UTF16);
-
-  // Check to make sure that the BOM is there and that the data size is correct
+    // Check to make sure that the BOM is there and that the data size is correct
 
+    const ByteVector utf16 = unicode.data(String::UTF16);
     CPPUNIT_ASSERT(utf16.size() == 2 + (unicode.size() * 2));
-
     CPPUNIT_ASSERT(unicode == String(utf16, String::UTF16));
   }
 
@@ -135,6 +170,21 @@ public:
     CPPUNIT_ASSERT_EQUAL(String(), String(b, String::UTF16));
   }
 
+  void testSurrogatePair()
+  {
+    // Make sure that a surrogate pair is converted into single UTF-8 char
+    // and vice versa.
+
+    const ByteVector v1("\xff\xfe\x42\xd8\xb7\xdf\xce\x91\x4b\x5c");
+    const ByteVector v2("\xf0\xa0\xae\xb7\xe9\x87\x8e\xe5\xb1\x8b");
+
+    const String s1(v1, String::UTF16);
+    CPPUNIT_ASSERT_EQUAL(s1.data(String::UTF8), v2);
+
+    const String s2(v2, String::UTF8);
+    CPPUNIT_ASSERT_EQUAL(s2.data(String::UTF16), v1);
+  }
+
   void testAppendStringDetach()
   {
     String a("a");
@@ -193,6 +243,85 @@ public:
     CPPUNIT_ASSERT_EQUAL(String("-123aa").toInt(), -123);
   }
 
+  void testSubstr()
+  {
+    CPPUNIT_ASSERT_EQUAL(String("01"), String("0123456").substr(0, 2));
+    CPPUNIT_ASSERT_EQUAL(String("12"), String("0123456").substr(1, 2));
+    CPPUNIT_ASSERT_EQUAL(String("123456"), String("0123456").substr(1, 200));
+  }
+
+  void testNewline()
+  {
+    ByteVector cr("abc\x0dxyz", 7);
+    ByteVector lf("abc\x0axyz", 7);
+    ByteVector crlf("abc\x0d\x0axyz", 8);
+
+    CPPUNIT_ASSERT_EQUAL(TagLib::uint(7), String(cr).size());
+    CPPUNIT_ASSERT_EQUAL(TagLib::uint(7), String(lf).size());
+    CPPUNIT_ASSERT_EQUAL(TagLib::uint(8), String(crlf).size());
+
+    CPPUNIT_ASSERT_EQUAL(L'\x0d', String(cr)[3]);
+    CPPUNIT_ASSERT_EQUAL(L'\x0a', String(lf)[3]);
+    CPPUNIT_ASSERT_EQUAL(L'\x0d', String(crlf)[3]);
+    CPPUNIT_ASSERT_EQUAL(L'\x0a', String(crlf)[4]);
+  }
+
+  void testEncode()
+  {
+    String jpn(L"\u65E5\u672C\u8A9E");
+    ByteVector jpn1 = jpn.data(String::Latin1);
+    ByteVector jpn2 = jpn.data(String::UTF8);
+    ByteVector jpn3 = jpn.data(String::UTF16);
+    ByteVector jpn4 = jpn.data(String::UTF16LE);
+    ByteVector jpn5 = jpn.data(String::UTF16BE);
+    std::string jpn6 = jpn.to8Bit(false);
+    std::string jpn7 = jpn.to8Bit(true);
+
+    CPPUNIT_ASSERT_EQUAL(ByteVector("\xE5\x2C\x9E"), jpn1);
+    CPPUNIT_ASSERT_EQUAL(ByteVector("\xE6\x97\xA5\xE6\x9C\xAC\xE8\xAA\x9E"), jpn2);
+    CPPUNIT_ASSERT_EQUAL(ByteVector("\xFF\xFE\xE5\x65\x2C\x67\x9E\x8A"), jpn3);
+    CPPUNIT_ASSERT_EQUAL(ByteVector("\xE5\x65\x2C\x67\x9E\x8A"), jpn4);
+    CPPUNIT_ASSERT_EQUAL(ByteVector("\x65\xE5\x67\x2C\x8A\x9E"), jpn5);
+    CPPUNIT_ASSERT_EQUAL(std::string("\xE5\x2C\x9E"), jpn6);
+    CPPUNIT_ASSERT_EQUAL(std::string("\xE6\x97\xA5\xE6\x9C\xAC\xE8\xAA\x9E"), jpn7);
+
+    String empty;
+    ByteVector empty1 = empty.data(String::Latin1);
+    ByteVector empty2 = empty.data(String::UTF8);
+    ByteVector empty3 = empty.data(String::UTF16);
+    ByteVector empty4 = empty.data(String::UTF16LE);
+    ByteVector empty5 = empty.data(String::UTF16BE);
+    std::string empty6 = empty.to8Bit(false);
+    std::string empty7 = empty.to8Bit(true);
+
+    CPPUNIT_ASSERT(empty1.isEmpty());
+    CPPUNIT_ASSERT(empty2.isEmpty());
+    CPPUNIT_ASSERT_EQUAL(ByteVector("\xFF\xFE"), empty3);
+    CPPUNIT_ASSERT(empty4.isEmpty());
+    CPPUNIT_ASSERT(empty5.isEmpty());
+    CPPUNIT_ASSERT(empty6.empty());
+    CPPUNIT_ASSERT(empty7.empty());
+  }
+
+  void testIterator()
+  {
+    String s1 = "taglib string";
+    String s2 = s1;
+
+    String::Iterator it1 = s1.begin();
+    String::Iterator it2 = s2.begin();
+
+    CPPUNIT_ASSERT_EQUAL(L't', *it1);
+    CPPUNIT_ASSERT_EQUAL(L't', *it2);
+
+    std::advance(it1, 4);
+    std::advance(it2, 4);
+    *it2 = L'I';
+    CPPUNIT_ASSERT_EQUAL(L'i', *it1);
+    CPPUNIT_ASSERT_EQUAL(L'I', *it2);
+  }
+
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestString);
+
